@@ -409,20 +409,16 @@ if not vars().has_key('BOUNCE_TEXT_FAIL_KEYWORD'):
 if not vars().has_key('BARE_APPEND'):
     BARE_APPEND = None
 
-# CGI_ACTIVE
-# All defaults beginning with "CGI_" configure the behavior of tmda-cgi.
-# tmda-cgi is provided as a contrib package for web-based monitoring and
-# configuration of TMDA accounts.  To use tmda-cgi, you must define
-# CGI_ACTIVE to enable default values for the CGI_* variables.
-#
-# Set this in /etc/tmdarc if you set up tmda-cgi in system-wide mode.
+# CGI_SETTINGS
+# Filename for saving various tmda-cgi settings.  This filename can
+# include a full path, otherwise it is relative the user's config file.
 #
 # Example:
-# CGI_ACTIVE = 1
+# CGI_SETTINGS = "/home/jim/.tmda/MyCGISettings"
 #
-# Default is 0 (disabled)
-if not vars().has_key('CGI_ACTIVE'):
-    CGI_ACTIVE = 0
+# Default is "tmda-cgi.ini".
+if not vars().has_key('CGI_SETTINGS'):
+    CGI_SETTINGS = "tmda-cgi.ini"
 
 # CGI_URL
 # Absolute URL of your tmda-cgi installation.
@@ -431,103 +427,8 @@ if not vars().has_key('CGI_ACTIVE'):
 # CGI_URL = "http://www.domain.dom/cgi-bin/tmda.cgi"
 #
 # No default.
-if CGI_ACTIVE and not vars().has_key('CGI_URL'):
+if not vars().has_key('CGI_URL'):
     CGI_URL = None
-    
-# CGI_CLEANUP_ODDS
-# Chance of cleaning up old sessions when a session object is
-# instantiated.  Keeping this value small will minimize overhead on
-# each page fetch.  Larger values will tidy up the temporary files
-# more often.
-#
-# You probably won't need to adjust this parameter.
-#
-# Example:
-# CGI_CLEANUP_ODDS = 0.1
-#
-# Default is 0.01 (1%).
-if CGI_ACTIVE and not vars().has_key('CGI_CLEANUP_ODDS'):
-    CGI_CLEANUP_ODDS = 0.01
-
-# CGI_CROP_SENDER
-# Maximum length of a sender's e-mail address before it will be cropped in
-# list view.
-#
-# Example:
-# CGI_CROP_SENDER = 40
-#
-# Default is 25.
-if CGI_ACTIVE and not vars().has_key('CGI_CROP_SENDER'):
-  CGI_CROP_SENDER = 25
-
-# CGI_CROP_SUBJECT
-# Maximum length of an e-mail subject before it will be cropped in list view.
-#
-# Example:
-# CGI_CROP_SUBJECT = 50
-#
-# Default is 40.
-if CGI_ACTIVE and not vars().has_key('CGI_CROP_SUBJECT'):
-  CGI_CROP_SUBJECT = 40
-
-# CGI_DATE_FORMAT
-# Format string for time.strftime() used when displaying message dates
-# in pending list view.
-#
-# Example:
-# CGI_DATE_FORMAT = "%m/%d/%y"
-#
-# Default is "%a %1m/%d" (which generates text like: "Mon 12/31")
-if CGI_ACTIVE and not vars().has_key('CGI_DATE_FORMAT'):
-    CGI_DATE_FORMAT = '%a %1m/%d'
-
-# CGI_PAGER_SIZE
-# Number of pending e-mails shown in list format on one page.
-#
-# Example:
-# CGI_PAGER_SIZE = 10
-#
-# Default is 25.
-if CGI_ACTIVE and not vars().has_key('CGI_PAGER_SIZE'):
-    CGI_PAGER_SIZE = 25
-
-# CGI_SESSION_EXP
-# Number of seconds a session is guaranteed to remain on the system
-# before there is a chance of it being cleaned up.  Users who are
-# logged into tmda-cgi and spend more than this amount of time between
-# page accesses may have to log in again.
-#
-# Example:
-# CGI_SESSION_EXP = 600
-#
-# Default is 300 (5 minutes).
-if CGI_ACTIVE and not vars().has_key('CGI_SESSION_EXP'):
-    CGI_SESSION_EXP = 300
-
-# CGI_USE_JS_CONFIRM
-# Enables a pop-up confirmation box before any message is deleted or
-# blacklisted.  Must be disabled if you will be surfing tmda-cgi in
-# a browser without Javascript or where Javascript is disabled.
-# Otherwise you will not be able to delete or blacklist messages.
-#
-# Example:
-# CGI_USE_JS_CONFIRM = 0
-#
-# Default is 1.
-if CGI_ACTIVE and not vars().has_key('CGI_USE_JS_CONFIRM'):
-    CGI_USE_JS_CONFIRM = 1
-
-# CGI_USER
-# Defines the user name to use at non-critical times (such as reading
-# and writing session files.  CGI_USER is only signifigant when running
-# in system-wide mode.  See tmda-cgi.html for more about modes.
-#
-# Example:
-# CGI_USER = "apache"
-#
-# Default is "nobody".
-if CGI_ACTIVE and not vars().has_key('CGI_USER'):
-    CGI_USER = "nobody"
 
 # CONFIRM_ADDRESS
 # An optional e-mail address to use for creating confirmation
@@ -752,6 +653,20 @@ if not vars().has_key('FILTER_BOUNCE_CC'):
 if not vars().has_key('FILTER_DROP_CC'):
     FILTER_DROP_CC = None
 
+# ACTION_HEADER_INCOMING
+# Set this variable to 1 if you want TMDA to add an `X-TMDA-Action'
+# header to your delivered incoming messages. The value of this header
+# is the same as the "Actn:" field in a LOGFILE_INCOMING entry. e.g,
+#
+# X-TMDA-Action: OK (from johndoe* ok)
+#
+# NOTE: This will not work if you are running qmail and have not set
+# the DELIVERY variable.
+#
+# Default is 0 (turned off)
+if not vars().has_key('ACTION_HEADER_INCOMING'):
+    ACTION_HEADER_INCOMING = 0
+
 # ACTION_INCOMING
 # Specifies how incoming messages should be disposed of by default if
 # they didn't match FILTER_INCOMING and were not sent to a tagged
@@ -1005,14 +920,6 @@ if not vars().has_key('LOGFILE_INCOMING'):
 if not vars().has_key('LOGFILE_OUTGOING'):
     LOGFILE_OUTGOING = None
 
-# LOCALDATE
-# Set this variable to 0 if you want TMDA to generate ``Date:''
-# headers in UTC instead of the local time zone.
-#
-# Default is 1 (use local time zone)
-if not vars().has_key('LOCALDATE'):
-    LOCALDATE = 1
-
 # MESSAGE_FROM_STYLE
 # Specifies how `From' and `Resent-From' headers should look when
 # tagging outgoing messages with tmda-sendmail.  There are two valid
@@ -1083,6 +990,62 @@ if not vars().has_key('RESPONSE_DIR') and MAX_AUTORESPONSES_PER_DAY != 0:
 if not vars().has_key('AUTORESPONSE_INCLUDE_SENDER_COPY'):
     AUTORESPONSE_INCLUDE_SENDER_COPY = 2
 
+# MYSQL_ENABLED
+# All defaults beginning with "MYSQL_" configure the use of a MySQL
+# database in the filtering process.  To use a MySQL database, you
+# must have the _mysql library loaded.  See:
+# http://sourceforge.net/projects/mysql-python/
+#
+# MYSQL_ENABLED must be set to 1 if you wish to use a MySQL database
+# in your filters.
+#
+# Example:
+# MYSQL_ENABLED = 1
+#
+# Default is 0 (disabled)
+if not vars().has_key('MYSQL_ENABLED'):
+    MYSQL_ENABLED = 0
+
+# MYSQL_HOST
+# MySQL host to connect to.
+#
+# Example:
+# MYSQL_HOST = "db.my.server"
+#
+# Default is "localhost".
+if MYSQL_ENABLED and not vars().has_key('MYSQL_HOST'):
+    MYSQL_HOST = "localhost"
+
+# MYSQL_DATABASE
+# MySQL database containing any lists used in filter.
+#
+# Example:
+# MYSQL_DATABASE = "JimsTMDA"
+#
+# Default is "TMDA".
+if MYSQL_ENABLED and not vars().has_key('MYSQL_DATABASE'):
+    MYSQL_DATABASE = "TMDA"
+
+# MYSQL_USER
+# MySQL user to use when connecting to database.
+#
+# Example:
+# MYSQL_USER = "Jim"
+#
+# Default is "TMDA".
+if MYSQL_ENABLED and not vars().has_key('MYSQL_USER'):
+    MYSQL_USER = "TMDA"
+
+# MYSQL_PASSWORD
+# MySQL password to use when connecting to database.
+#
+# Example:
+# MYSQL_USER = "G@rfi31d"
+#
+# Default is "".
+if MYSQL_ENABLED and not vars().has_key('MYSQL_PASSWORD'):
+    MYSQL_PASSWORD = ""
+
 # PENDING_CACHE
 # Path to the cache file used when tmda-pending is invoked with the
 # --cache option.
@@ -1248,7 +1211,8 @@ if not vars().has_key('PURGED_HEADERS'):
 # The header should contain one fully-qualified e-mail address.  This
 # can be used in rare cases when you need to override the RECIPIENT
 # environment variable, which is how TMDA normally determines the
-# envelope recipient address.
+# envelope recipient address. If EXT isn't set, the address extension
+# will also be extracted from RECIPIENT_HEADER.
 #
 # Example:
 #
@@ -1339,6 +1303,28 @@ if not vars().has_key('TAGS_SENDER'):
 if not vars().has_key('TERSE_SUMMARY_HEADERS'):
     TERSE_SUMMARY_HEADERS = ['from_name', 'subject']
 
+# TMDAINJECT
+# A string containing one or more of the following letters which toggle
+# on certain behaviors in tmda-inject (which is used by both
+# tmda-sendmail and tmda-ofmipd):
+#
+# 'd' - Add a new Date field to the message, clobbering any
+#       existing Date fields.  Normally tmda-inject only adds a
+#       Date field if the incoming message lacks one.
+#
+# 'i' - Add a new Message-ID field to the message, clobbering any
+#       existing Message-ID fields.  Normally tmda-inject only adds a
+#       Message-ID field if the incoming message lacks one.
+#
+# Examples:
+#
+# TMDAINJECT = "i"
+# TMDAINJECT = "di"
+#
+# No default.
+if not vars().has_key('TMDAINJECT'):
+    TMDAINJECT = ''
+
 # SUMMARY_HEADERS
 # A list containing one or more message headers that should be
 # displayed by tmda-pending's interactive mode. Listed headers are
