@@ -2,6 +2,9 @@
 #
 # Copyright (C) 2001,2002 Jason R. Mastaler <jason@mastaler.com>
 #
+# Authors: Tim Legant <tim@catseye.net> and
+#          Jason R. Mastaler <jason@mastaler.com>
+#
 # This file is part of TMDA.
 #
 # TMDA is free software; you can redistribute it and/or modify it
@@ -27,11 +30,12 @@ Filter file syntax documented in htdocs/config-filter.html
 
 
 import os
+import popen2
 import re
 import string
 import sys
-import types
 import time
+import types
 
 import Util
 
@@ -68,7 +72,6 @@ class MatchError(Error):
 
 
 class Macro:
-
     """Macro definition as parsed by the filter parser."""
 
     macro_words = re.compile(r'([_a-zA-Z][_\w]*)')
@@ -205,7 +208,6 @@ class Macro:
 
 
 class _FilterFile:
-    
     """Storage for per-file data.  Used internally by FilterParser."""
 
     def __init__(self, filename):
@@ -219,8 +221,8 @@ class FilterParser:
     bol_comment = re.compile(r'\s*#')
 
     most_sources = re.compile(r"""
-    ( (?:to|from)-(?:file|cdb|dbm|ezmlm|mailman)
-    | size
+    ( (?:to|from)-(?:file|cdb|dbm|ezmlm|mailman|mysql)
+    | size | pipe
     | (?:to|from) (?!-) )
     """, re.VERBOSE | re.IGNORECASE)
 
@@ -260,23 +262,29 @@ class FilterParser:
     arguments = {
         'from'         : None,
         'to'           : None,
-        'from-file'    : ('autocdb', 'autodbm', 'optional'),
-        'to-file'      : ('autocdb', 'autodbm', 'optional'),
-        'from-cdb'     : ('optional',),
-        'to-cdb'       : ('optional',),
-        'from-dbm'     : ('optional',),
-        'to-dbm'       : ('optional',),
+        'from-file'    : ('autocdb', 'autodbm', 'domains', 'optional'),
+        'to-file'      : ('autocdb', 'autodbm', 'domains', 'optional'),
+        'from-cdb'     : ('domains', 'optional',),
+        'to-cdb'       : ('domains', 'optional',),
+        'from-dbm'     : ('domains', 'optional',),
+        'to-dbm'       : ('domains', 'optional',),
         'from-ezmlm'   : ('optional',),
         'to-ezmlm'     : ('optional',),
         'from-mailman' : ('attr', 'optional' ),
         'to-mailman'   : ('attr', 'optional' ),
+        'from-mysql'   : ('like', 'rlike'),
+        'to-mysql'     : ('like', 'rlike'),
         'body'         : ('case',),
         'headers'      : ('case',),
         'body-file'    : ('case', 'optional'),
         'headers-file' : ('case', 'optional'),
-        'size'         : None
+        'size'         : None,
+        'pipe'         : None
         }
 
+    # MySQL connection
+    MySQL = None
+
 
     def __init__(self):
         self.macros = []
@@ -578,7 +586,7 @@ class FilterParser:
 	Parse a single rule from a filter file.  If successful, return a tuple
 	with five fields.  The five fields are:
 	
-	  source    - string: to*, from*, body*, headers*, size
+	  source    - string: to*, from*, body*, headers*, size, pipe
           args      - any arguments that might be specified
 	  match     - string: the email address to be matched against, a
                       filename or a regular expression enclosed within
@@ -706,6 +714,44 @@ class FilterParser:
         return found_match
 
 
+    def __search_mysql(self, Table, Args, Keys, Actions, Source):
+        "Search MySQL table."
+        if not self.MySQL:
+            # Connect to the database if we have not yet connected
+            import Defaults, _mysql
+            self.MySQL = _mysql.connect \
+            (
+                host = Defaults.MYSQL_HOST,
+                db = Defaults.MYSQL_DATABASE,
+                user = Defaults.MYSQL_USER,
+                passwd = Defaults.MYSQL_PASSWORD
+            )
+        # Searches can be for records that are "=", "like", or "rlike" the
+        # keys.  "=" is the fastest and the default, but it does not allow any
+        # wildcarding.  "like" allows "_" to mean any character and "%" to
+        # mean any string.  "rlike" allows regular expressions.
+        Compare = "="
+        if Args.has_key('like'): Compare = "LIKE"
+        elif Args.has_key('rlike'): Compare = "RLIKE"
+        Test = ""
+        for Key in Keys:
+            if Test: Test += "OR "
+            Test += "('%s' %s ADDRESS) " % (Key, Compare)
+        # Perform the query
+        self.MySQL.query("SELECT * FROM %s WHERE %s LIMIT 1" % (Table, Test))
+        # Fetch the result
+        Result = self.MySQL.store_result().fetch_row(0, 1)
+        found_match = len(Result)
+
+        # If there is an entry for this key, we consider it an overriding
+        # action specification.
+        if found_match and Result[0]["ACTION"]:
+            Actions.clear()
+            Actions.update(self.__buildactions(Result[0]["ACTION"], Source))
+
+        return found_match
+
+
     def __search_cdb(self, pathname, keys, actions, source):
         """
         Search DJB's constant databases; see <http:/cr.yp.to/cdb.html>.
@@ -783,6 +829,19 @@ class FilterParser:
         return (dbname, search_func)
 
 
+    def __extract_domains(self, keys):
+        """
+        Attempt to extract the domain name from each address in keys.
+        """
+        domains = []
+        for k in keys:
+            try:
+                domains.append(k.split('@', 1)[1])
+            except IndexError:
+                pass
+        return domains
+     
+
     def firstmatch(self, recipient, senders=None,
                    msg_body=None, msg_headers=None, msg_size=None):
         """Iterate over each rule in the list looking for a match.  As
@@ -798,7 +857,6 @@ class FilterParser:
                 keys = senders
             elif source.startswith('to') and recipient:
                 keys = [recipient]
-            #
             # Here starts the matching against the various rules
             #
             # regular 'from' or 'to' addresses
@@ -810,6 +868,8 @@ class FilterParser:
             if source in ('from-file', 'to-file'):
                 dbname = os.path.expanduser(match)
                 search_func = self.__search_file
+                if args.has_key('domains'):
+                    keys += self.__extract_domains(keys)
                 # If we have an 'auto*' argument, ensure that the database
                 # is up-to-date.  If the 'optional' argument is also given,
                 # don't die if the file doesn't exist.
@@ -837,6 +897,8 @@ class FilterParser:
             if source in ('from-dbm', 'to-dbm'):
                 import anydbm
                 match = os.path.expanduser(match)
+                if args.has_key('domains'):
+                    keys += self.__extract_domains(keys)
                 try:
                     found_match = self.__search_dbm(match, keys,
                                                     actions, source)
@@ -849,6 +911,8 @@ class FilterParser:
             if source in ('from-cdb', 'to-cdb'):
                 import cdb
                 match = os.path.expanduser(match)
+                if args.has_key('domains'):
+                    keys += self.__extract_domains(keys)
                 try:
                     found_match = self.__search_cdb(match, keys,
                                                     actions, source)
@@ -918,6 +982,39 @@ class FilterParser:
                             break
                 if found_match:
 		    break
+            # MySQL-style databases.
+            if source in ('from-mysql', 'to-mysql'):
+                found_match = self.__search_mysql \
+                (
+                    match, args, keys, actions, source
+                )
+                if found_match:
+		    break
+            # A match is found if the command exits with a zero exit
+            # status.
+            if source == 'pipe' and msg_body and msg_headers:
+                cmd = popen2.Popen3(match, 1, bufsize=-1)
+                cmdout, cmdin, cmderr = cmd.fromchild, cmd.tochild, cmd.childerr
+                cmdin.write(msg_headers + '\n' + msg_body)
+                cmdin.flush()
+                cmdin.close()
+                err = cmderr.read().strip()
+                cmderr.close()
+                out = cmdout.read().strip()
+                cmdout.close()
+                r = cmd.wait()
+                if r == 0:
+                    found_match = 1
+                    break
+                else:
+                    # non-zero exit status
+                    if os.WIFEXITED(r):
+                        pass
+                    # raise an exception if the process exited due to
+                    # a signal.
+                    elif os.WIFSIGNALED(r):
+                        raise Error, 'command "%s" abnormal exit signal %s (%s)' \
+                              % (match, os.WTERMSIG(r), err or '')
             if source in ('body', 'headers'):
                 if source == 'body' and msg_body:
                     content = msg_body
