@@ -22,7 +22,11 @@
 """General purpose functions."""
 
 
+from cStringIO import StringIO
+from email.Generator import Generator
+from string import whitespace as WHITESPACE
 import cPickle
+import email
 import email.Utils
 import fileinput
 import fnmatch
@@ -40,14 +44,18 @@ import time
 import Errors
 
 
-EMPTYSTRING = ''
-NL = '\n'
 DOT = '.'
-from string import whitespace as WHITESPACE
+EMPTYSTRING = ''
+MODE_EXEC = 01
+MODE_READ = 04
+MODE_WRITE = 02
+NL = '\n'
 
 
 def gethostname():
-    hostname = os.environ.get('QMAILHOST') or \
+    """The host name"""
+    hostname = os.environ.get('TMDAHOST') or \
+               os.environ.get('QMAILHOST') or \
                os.environ.get('MAILHOST')
     if not hostname:
         hostname = socket.getfqdn()
@@ -55,18 +63,22 @@ def gethostname():
 
 
 def getfullname():
-    fullname = os.environ.get('QMAILNAME') or \
+    """The user's personal name"""
+    fullname = os.environ.get('TMDANAME') or \
+               os.environ.get('QMAILNAME') or \
                os.environ.get('NAME') or \
                os.environ.get('MAILNAME')
     if not fullname:
         fullname = pwd.getpwuid(os.getuid())[4]
     if not fullname:
-        fullname = ''
+        fullname = 'Jane Doe'
     return fullname
 
 
 def getusername():
-    username = os.environ.get('QMAILUSER') or \
+    """The user name"""
+    username = os.environ.get('TMDAUSER') or \
+               os.environ.get('QMAILUSER') or \
                os.environ.get('USER') or \
                os.environ.get('LOGNAME')
     if not username:
@@ -117,6 +129,48 @@ def getfileuid(path):
     return statinfo[stat.ST_UID]
 
 
+def getvuserhomedir(user, domain, script):
+    """Return the home directory of a qmail virtual domain user."""
+    cmd = "%s %s %s" % (script, user, domain)
+    fpin = os.popen(cmd)
+    vuserhomedir = fpin.read()
+    fpin.close()
+    return vuserhomedir.strip()
+
+
+def getuserparams(login):
+    "Return a user's home directory, UID, & GID."
+
+    stats = pwd.getpwnam(login)
+    return (stats[5], stats[2], stats[3])
+
+
+def RunTask(Args):
+    """Run a program the "hard way" so we don't lose our UID."""
+
+    # Open a pipe between the parent and a child process
+    Read, Write = os.pipe()
+    if not os.fork():
+        # Child writes only and can close the reader
+        os.close(Read)
+
+        # Capture the STDOUT and stick it in the pipe
+        os.dup2(Write, 1)
+
+        # Launch the program
+        os.execv(Args[0], Args)
+
+    # Parent reads only and can close the writer
+    os.close(Write)
+
+    # Capture contents of pipe
+    Read = os.fdopen(Read)
+    RetVal = Read.readlines()
+    Read.close()
+
+    return RetVal
+
+
 def seconds(timeout):
     """Translate the defined timeout interval into seconds."""
     match = re.match("^([0-9]+)([YMwdhms])$", timeout)
@@ -184,7 +238,7 @@ def unixdate(timesecs=None):
 def make_msgid(timesecs=None, pid=None):
     """Return an rfc2822 compliant Message-ID: string, composed of
     seconds since the epoch in UTC + process id + 'TMDA' + FQDN. e.g:
-    
+
     <1016659379.10104.TMDA@nightshade.la.mastaler.com>
 
     timesecs is optional, and if not given, the current time is used.
@@ -196,7 +250,9 @@ def make_msgid(timesecs=None, pid=None):
     if not pid:
         import Defaults
         pid = Defaults.PID
-    idhost = os.environ.get('QMAILIDHOST') or gethostname()
+    idhost = os.environ.get('TMDAIDHOST') or \
+             os.environ.get('QMAILIDHOST') or \
+             gethostname()
     msgid = '<%s.%s.TMDA@%s>' % (int(timesecs), pid, idhost)
     return msgid
 
@@ -204,14 +260,15 @@ def make_msgid(timesecs=None, pid=None):
 def make_date(timesecs=None):
     """Return an RFC 2822 compliant Date: string.  e.g,
 
-    Thu, 16 May 2002 04:23:10 +1200
-    
+    Thu, 24 Apr 2003 10:15:15 +1200 (NZST)
+
     timesecs is optional, and if not given, the current time is used.
     """
     if timesecs is None:
         timesecs = time.time()
-    from Defaults import LOCALDATE
-    return email.Utils.formatdate(timesecs, LOCALDATE)
+    tzname = time.tzname[time.localtime(timesecs)[-1]]
+    basedate = email.Utils.formatdate(timesecs, localtime=1)
+    return '%s (%s)' % (basedate, tzname)
 
 
 def file_to_dict(file, dict):
@@ -254,7 +311,7 @@ def pipecmd(command, *strings):
     *strings are optional pieces of data to write to command.
 
     return_status will just return the exit status of the command..
-    
+
     Based on code from getmail
     <URL:http://www.qcc.sk.ca/~charlesc/software/getmail-2.0/>
     Copyright (C) 2001 Charles Cazabon, and licensed under the GNU
@@ -312,7 +369,7 @@ def writefile(contents, fullpathname):
         file = open(fullpathname, 'w')
         file.write(contents)
         file.close()
-        
+
 
 def append_to_file(str, fullpathname):
     """Append a string to a text file if it isn't already in there."""
@@ -411,12 +468,35 @@ def confirm_append_address(xp, rp):
     return rp
 
 
+def msg_as_string(msg, maxheaderlen=0, mangle_from_=0, unixfrom=0):
+    """A more flexible replacement for Message.as_string().  The default
+    is a textual representation of the message where the headers are
+    not wrapped, From is not escaped, and a leading From_ line is not
+    added.
+
+    msg is an email.Message.Message object.
+
+    maxheaderlen specifies the longest length for a non-continued
+    header.  Disabled by default.  RFC 2822 recommends 78.
+    
+    mangle_from_ escapes any line in the body that begins with "From"
+    with ">".  Useful when writing to Unix mbox files.  Default is
+    False.
+
+    unixfrom forces the printing of the envelope header delimiter.
+    Default is False."""
+    fp = StringIO()
+    g = Generator(fp, mangle_from_=mangle_from_, maxheaderlen=maxheaderlen)
+    g.flatten(msg, unixfrom=unixfrom)
+    return fp.getvalue()
+
+
 def sendmail(msgstr, envrecip, envsender):
     """Send e-mail via direct SMTP, or by opening a pipe to the
     sendmail program.
-    
+
     msgstr is an rfc2822 message as a string.
-    
+
     envrecip is the envelope recipient address.
 
     envsender is the envelope sender address.
@@ -460,29 +540,36 @@ def headers_as_list(msg):
     return ['%s: %s' % (k, v) for k, v in msg.items()]
 
 
-def headers_as_string(msg):
-    """Return the (decoded) message headers as a string."""
-    return '\n'.join(['%s: %s' %
-                      (k, decode_header(v)) for k, v in msg.items()])
-
-
 def headers_as_raw_string(msg):
     """Return the headers as a raw (undecoded) string."""
-    msgtext = msg.as_string()
+    msgtext = msg_as_string(msg)
     idx = msgtext.index('\n\n')
     return msgtext[:idx+1]
 
 
+def headers_as_string(msg):
+    """Return the (decoded) message headers as a string.  If the
+    sequence can't be decoded, punt and return a raw (undecoded)
+    string instead."""
+    try:
+        hdrstr = '\n'.join(['%s: %s' %
+                            (k, decode_header(v)) for k, v in msg.items()])
+    except email.Errors.HeaderParseError:
+        hdrstr = headers_as_raw_string(msg)
+    return hdrstr
+
+
 def body_as_raw_string(msg):
     """Return the body as a raw (undecoded) string."""
-    msgtext = msg.as_string()
+    msgtext = msg_as_string(msg)
     idx = msgtext.index('\n\n')
     return msgtext[idx+2:]
 
 
+
 def rename_headers(msg, old, new):
     """Rename all occurances of a message header in a Message object.
-        
+
     msg is an email.Message.Message object.
 
     old is name of the header to rename.
@@ -692,7 +779,7 @@ def maketext(templatefile, vardict):
     The first match found stops the search.  In this way, you can
     specialize templates at the desired level, or, if you use only the
     default templates, you don't need to change anything.
-    
+
     Once the templatefile is found, string substitution is performed
     by interpolation in `localdict'.
 
@@ -735,11 +822,11 @@ def maketext(templatefile, vardict):
         localdict.update(vardict)
         text = template % localdict
         return text
-        
+
 
 def filter_match(filename, recip, sender=None):
     """Check if the give e-mail addresses match lines in filename."""
-    import FilterParser 
+    import FilterParser
     filter = FilterParser.FilterParser()
     filter.read(filename)
     (actions, matchline) = filter.firstmatch(recip, [sender])
@@ -756,3 +843,97 @@ def filter_match(filename, recip, sender=None):
         print 'MATCH:', matchline
     else:
         print 'Sorry, no matching lines.'
+
+
+def CanRead( file, uid = None, gid = None, raiseError = 1 ):
+    try:
+        return CanMode( file, MODE_READ, uid, gid )
+    except IOError:
+        if not raiseError:
+            return 0
+        else:
+            pass
+
+
+def CanWrite( file, uid = None, gid = None, raiseError = 1 ):
+    try:
+        return CanMode( file, MODE_WRITE, uid, gid )
+    except IOError:
+        if not raiseError:
+            return 0
+        else:
+            pass
+
+
+def CanExec( file, uid = None, gid = None, raiseError = 1 ):
+    try:
+        return CanMode( file, MODE_EXEC, uid, gid )
+    except IOError:
+        if not raiseError:
+            return 0
+        else:
+            pass
+
+
+def CanMode( file, mode = MODE_READ, uid = None, gid = None ):
+    try:
+        fstat = os.stat( file )
+    except:
+        raise IOError, "'%s' does not exist" % file
+    if uid is None:
+        uid = os.geteuid()
+    if gid is None:
+        gid = os.getegid()
+    needuid = fstat.st_uid
+    needgid = fstat.st_gid
+    filemod = fstat.st_mode & 0777
+    if uid == 0:
+        # Root always wins.
+        return 1
+    elif filemod & mode:
+        return 1
+    elif filemod & ( mode * 010 ) and needgid == gid:
+        return 1
+    elif filemod & ( mode * 0100 ) and needuid == uid:
+        return 1
+    else:
+        return 0
+
+
+class DevnullOutput:
+    def write(self, msg): pass
+    def flush(self): pass
+    def __repr__(self):
+        return ""
+
+
+class StringOutput:
+    def __init__(self):
+        self.__content = ""
+    def write(self, msg):
+        if msg != "":
+            self.__content += "%s" % msg
+    def flush(self):
+        self.__content = ""
+    def __repr__(self):
+        return self.__content
+
+
+class Debugable:
+    def __init__(self, outputObject = DevnullOutput() ):
+        self.DEBUGSTREAM = outputObject
+        if self.DEBUGSTREAM is DevnullOutput:
+            self.level = 0
+        else:
+            self.level = 1
+
+    def debug(self, msg, level = 1):
+        if self.level >= level:
+            print >> self.DEBUGSTREAM, msg
+
+    def set_debug(level = 1):
+        self.level = level
+
+    def set_nodebug():
+        self.level = 0
+
